package com.ruoyi.web.backgroud.client;

import com.ruoyi.web.config.ConfigComponent;
import io.netty.bootstrap.Bootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioSocketChannel;
import io.netty.handler.codec.string.StringDecoder;
import io.netty.handler.codec.string.StringEncoder;
import io.netty.handler.timeout.IdleStateHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.Resource;
import java.util.concurrent.TimeUnit;

@Slf4j
@Configuration
public class BackgroundClientNetty {

    @Resource
    ConfigComponent component;
    Bootstrap bootstrap;
    Channel channel;

    @Bean("backgroundClientHandler")
    public BackgroundClientHandler backgroundClientHandler() {
        return new BackgroundClientHandler(this);
    }

    @Bean("backgroundClient")
    public Bootstrap bootstrap() {
        EventLoopGroup group = new NioEventLoopGroup();
        bootstrap = new Bootstrap();
        return bootstrap.group(group)
                .channel(NioSocketChannel.class)
                .remoteAddress(component.getHost(), component.getBeatPort())
                .handler(new ChannelInitializer<SocketChannel>() {
                    @Override
                    protected void initChannel(SocketChannel socketChannel) throws Exception {
                        socketChannel.pipeline()
                                .addLast("decoder", new StringDecoder())
                                .addLast("encoder", new StringEncoder())
                                .addLast(new IdleStateHandler(0, 5, 0))
                                .addLast();
                    }
                });
    }


    public void connect() {
        ChannelFuture future = bootstrap().connect();
        future.addListener((ChannelFutureListener) item -> {
            if(item.isSuccess()) {
                channel = item.channel();
                log.info("-----------后端服务器连接成功,{},{}-----------",component.getHost(), component.getBeatPort());
            } else {
                log.error("-----------后端服务器连接失败,{},{}-----------进行重连",component.getHost(), component.getBeatPort());
                item.channel().eventLoop().schedule(this::connect, 5, TimeUnit.SECONDS);
            }
        });
        try {
            future.channel().closeFuture().sync();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    public void send(String message) {
        if(channel != null && channel.isActive()) {
            channel.writeAndFlush(message);
        } else {
            log.error("未建立连接，无法发送消息！");
        }
    }

    public void close() {
        if(channel != null) {
            channel.close();
        }
    }

}
