package com.ruoyi.web.service;

import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.web.bo.ResponseBo;
import com.ruoyi.web.config.ConfigComponent;
import com.ruoyi.web.config.RestUtil;
import com.ruoyi.web.entity.*;
import com.ruoyi.web.config.cache.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.util.ConcurrentReferenceHashMap;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

/**
 * 散货系统接口调用-业务处理
 */
@Service
@Slf4j
public class BulkloadService {

    //private static HttpHeaders headers = new HttpHeaders();
    private final static Map<String,HttpHeaders> HEADERS_MAP=new ConcurrentReferenceHashMap<>();
    //private final static Map<String,String> THRFLAG_MAP=new ConcurrentReferenceHashMap<>();
    @Autowired
    private ConfigComponent configComponent;

    /** 登录接口 */
    public ResponseBo LogOn(GatherData gatherData) {
        HttpHeaders headers = new HttpHeaders();
        gatherData.setJsStatus(0);
        ResponseBo res = new ResponseBo();
        res.setJsStatus(gatherData.getJsStatus());
        res.setContainerStatus(gatherData.getContainerStatus());
        String UserName = configComponent.getUserName();
        String PassWord = configComponent.getPassWord();

        String AuthToken = configComponent.getAuthToken();
        /*HMac mac = new HMac(HmacAlgorithm.HmacSHA256, UserName.getBytes());
        AuthToken = mac.digestBase64(PassWord, false);*/

        //请求地址
        String url = configComponent.getLogOnUrl();
        //请求体
        String body = "{\"UserName\":\""+UserName+"\",\"AuthToken\":\""+AuthToken+"\"}";
        JSONObject params = JSONObject.parseObject(body);

       // headers.clear();
        headers.setContentType(MediaType.parseMediaType(MediaType.APPLICATION_JSON_VALUE));
        headers.add("Accept", MediaType.APPLICATION_JSON_VALUE);
        String AccessToken = "";
        try {
            JSONObject result = RestUtil.post(url, headers, params);
            AccessToken = result.getString("AccessToken");
        } catch (Exception e) {
            String message = e.getMessage();
            if(message.contains("Message")) {
                message = message.substring(message.indexOf("Message\":\"")+10,message.indexOf("\"}]"));
            }
            log.info("登录失败！:{}",message);
            res.setCode(500);
            res.setMsg("登录失败！"+message);
            return res;
        }
        headers.add("authorization","Bearer " + AccessToken);
        HEADERS_MAP.put(gatherData.getVE_LICENSE_NO(),headers);
        log.info("登陆接口AccessToken={}",AccessToken);
        if(StringUtils.isEmpty(AccessToken)) {
            res.setCode(500);
            res.setMsg("登录失败！");
            return res;
        }
        res.setCode(200);
        if("I".equalsIgnoreCase(gatherData.getI_E_FLAG())) {//进港
            res = this.InVerification(gatherData);
        } else {//出港
            //res = this.OutWeight(gatherData,THRFLAG_MAP.get(gatherData.getVE_LICENSE_NO()));
            res = this.OutWeight(gatherData,RedisUtil.get("数据交换中心:"+gatherData.getVE_LICENSE_NO()));
        }
        return res;
    }

    /** 登出接口 */
    public void LogOff(String carNo) {
        //请求地址
        String url = configComponent.getLogOffUrl();
        try{
          HttpHeaders headers=  HEADERS_MAP.get(carNo);
           RestUtil.post(url, headers);
           HEADERS_MAP.remove(carNo);
        }catch (Exception e){
            log.error("login off error:{}",e.getMessage());
            HEADERS_MAP.remove(carNo);
        }

    }

    /** 进港：预约确认接口 */
    public ResponseBo InVerification(GatherData gatherData) {
        ResponseBo res = new ResponseBo();
        res.setJsStatus(gatherData.getJsStatus());
        res.setContainerStatus(gatherData.getContainerStatus());

        //请求地址：接口是否发送——散货过磅地点:默认0=不发送,1=发送
        String url = configComponent.getInVerificationUrl() + "?TruckNo=" + gatherData.getVE_LICENSE_NO();
        if(gatherData.getIsSendPlaceCode() == 1) {
            url = configComponent.getInVerificationUrl() + "?TruckNo=" + gatherData.getVE_LICENSE_NO() + "&PLACE_CODE=" + gatherData.getPlaceCode();
        }

        log.info("进港：预约确认接口url:{}",url);
        VerificationRes verificationRes = new VerificationRes();
        try {
            HttpHeaders headers=  HEADERS_MAP.get(gatherData.getVE_LICENSE_NO());
            JSONObject result = RestUtil.get(url, headers);
            log.info("进港：预约确认接口返回结果:{}",result.toString());
            verificationRes = JSONObject.parseObject(result.toString(), VerificationRes.class);
            if("false".equalsIgnoreCase(verificationRes.getRESULT())) {
                if(verificationRes.getMESSAGE().contains("未找到车辆") && verificationRes.getMESSAGE().contains("预约信息")) {
                    res.setFlag("散货无预约");
                }
                res.setCode(500);
                res.setMsg(verificationRes.getMESSAGE());
                //退出登录
                this.LogOff(gatherData.getVE_LICENSE_NO());
                return res;
            }
        } catch (Exception e) {
            String message = e.getMessage();
            if(message.contains("Message")) {
                message = message.substring(message.indexOf("Message\":\"")+10,message.indexOf("\"}]"));
            }
            log.info("进港：预约确认接口报错:{}",message);
            res.setCode(500);
            res.setMsg(message);
            return res;
        }
        res.setCode(200);
        res.setMsg(verificationRes.getMESSAGE());
        RedisUtil.set("数据交换中心:"+gatherData.getVE_LICENSE_NO(), verificationRes.getTHRFLAG());
        //THRFLAG_MAP.put(gatherData.getVE_LICENSE_NO(), verificationRes.getTHRFLAG());
        res = this.InWeight(gatherData,verificationRes.getTHRFLAG());
        return res;
    }

    /** 出港：预约确认接口 */
    public ResponseBo OutVerification(GatherData gatherData) {
        ResponseBo res = new ResponseBo();
        res.setJsStatus(gatherData.getJsStatus());
        res.setContainerStatus(gatherData.getContainerStatus());

        //请求地址：接口是否发送——散货过磅地点:默认0=不发送,1=发送
        String url = configComponent.getOutVerificationUrl() + "?TruckNo=" + gatherData.getVE_LICENSE_NO();
        if(gatherData.getIsSendPlaceCode() == 1) {
            url = configComponent.getOutVerificationUrl() + "?TruckNo=" + gatherData.getVE_LICENSE_NO() + "&PLACE_CODE=" + gatherData.getPlaceCode();
        }

        log.info("出港：预约确认接口url:{}",url);
        VerificationRes verificationRes = new VerificationRes();
        try {
            HttpHeaders headers=  HEADERS_MAP.get(gatherData.getVE_LICENSE_NO());
            JSONObject result = RestUtil.get(url, headers);
            log.info("出港：预约确认接口返回结果:{}",result.toString());
            verificationRes = JSONObject.parseObject(result.toString(), VerificationRes.class);
        } catch (Exception e) {
            String message = e.getMessage();
            if(message.contains("Message")) {
                message = message.substring(message.indexOf("Message\":\"")+10,message.indexOf("\"}]"));
            }
            log.info("出港：预约确认接口报错:{}",message);
            res.setCode(500);
            res.setMsg(message);
            return res;
        }

        res.setCode(200);
        res.setMsg(verificationRes.getMESSAGE());
        if("false".equalsIgnoreCase(verificationRes.getRESULT())) {
            res.setCode(500);
            res.setFlag("散货无预约");
            //退出登录
            this.LogOff(gatherData.getVE_LICENSE_NO());
            return res;
        }
        res = this.OutWeight(gatherData,verificationRes.getTHRFLAG());
        return res;
    }

    /** 进港：卡车称重接口 */
    public ResponseBo InWeight(GatherData gatherData, String THIRDFLAG) {
        ResponseBo res = new ResponseBo();
        res.setJsStatus(gatherData.getJsStatus());
        res.setContainerStatus(gatherData.getContainerStatus());
        //车牌号(必输)
        String TRUCK_NO = gatherData.getVE_LICENSE_NO();
        //过磅地点(必输)
        String PLACE_CODE = gatherData.getPlaceCode();
        //过磅重量NUMBER(12,4)(必输)
        //读取到的地磅重量——>KG转换为吨
        BigDecimal grossWt = new BigDecimal(gatherData.getGROSS_WT());
        grossWt = grossWt.divide(BigDecimal.valueOf(1000),2, RoundingMode.HALF_UP);
        double WEIGHT = grossWt.doubleValue();
        //时间戳
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        String UCINO = sdf.format(new Date());

        //请求地址
        String url = configComponent.getInWeightUrl();
        //请求体
        String body = "{\"TRUCK_NO\": \""+TRUCK_NO+"\", \"PLACE_CODE\": \""+PLACE_CODE+"\", \"WEIGHT\":"+WEIGHT+", \"THR_FLAG\":\""+THIRDFLAG+"\"}";

        JSONObject params = JSONObject.parseObject(body);
        log.info("进港：卡车称重接口url:{}",url);
        log.info("进港：卡车称重接口params:{}",params);
        VerificationRes verificationRes = new VerificationRes();
        try {
            //{"MESSAGE":"一次过磅称重成功","RESULT":true}
            HttpHeaders headers=  HEADERS_MAP.get(gatherData.getVE_LICENSE_NO());
            JSONObject jsonObject = RestUtil.post(url, headers, params);
            log.info("进港：卡车称重接口返回结果:{}",jsonObject);
            verificationRes = JSONObject.parseObject(jsonObject.toString(), VerificationRes.class);
            if("false".equalsIgnoreCase(verificationRes.getRESULT())) {
                res.setCode(500);
                res.setMsg(verificationRes.getMESSAGE());
                //退出登录
                this.LogOff(gatherData.getVE_LICENSE_NO());
                return res;
            }
            if(jsonObject != null && jsonObject.toString().contains("ErrorCode")) {
                res.setCode(500);
                res.setMsg(jsonObject.getString("Message"));
                //退出登录
                this.LogOff(gatherData.getVE_LICENSE_NO());
                return res;
            }
        } catch (Exception e) {
            String message = e.getMessage();
            if(message.contains("Message")) {
                message = message.substring(message.indexOf("Message\":\"")+10,message.indexOf("\"}]"));
            }
            log.info("进港：卡车称重接口报错:{}",message);
            res.setCode(500);
            res.setMsg(message);
            return res;
        }

        res.setCode(200);
        //res = this.GetInReport(gatherData,THIRDFLAG);
        res.setJsStatus(1);
        res.setMsg("散货接口通过");
        //退出登录
        this.LogOff(gatherData.getVE_LICENSE_NO());
        return res;
    }

    /** 出港：卡车称重接口 */
    public ResponseBo OutWeight(GatherData gatherData, String THIRDFLAG) {
        ResponseBo res = new ResponseBo();
        res.setJsStatus(gatherData.getJsStatus());
        res.setContainerStatus(gatherData.getContainerStatus());
        //车牌号(必输)
        String TRUCK_NO = gatherData.getVE_LICENSE_NO();
        //过磅地点(必输)
        String PLACE_CODE = gatherData.getPlaceCode();
        //过磅重量NUMBER(12,4)(必输)
        //读取到的地磅重量——>KG转换为吨
        BigDecimal grossWt = new BigDecimal(gatherData.getGROSS_WT());
        grossWt = grossWt.divide(BigDecimal.valueOf(1000),2, RoundingMode.HALF_UP);
        double WEIGHT = grossWt.doubleValue();
        //时间戳
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        String UCINO = sdf.format(new Date());

        //请求地址
        String url = configComponent.getOutWeightUrl();
        //请求体
        String body = "{\"TRUCK_NO\": \""+TRUCK_NO+"\", \"PLACE_CODE\": \""+PLACE_CODE+"\", \"WEIGHT\":"+WEIGHT+", \"THR_FLAG\":\""+THIRDFLAG+"\"}";

        JSONObject params = JSONObject.parseObject(body);
        log.info("出港：卡车称重接口url:{}",url);
        log.info("出港：卡车称重接口params:{}",params);
        HttpHeaders headers=  HEADERS_MAP.get(gatherData.getVE_LICENSE_NO());
        VerificationRes verificationRes = new VerificationRes();
        try {
            JSONObject jsonObject = RestUtil.post(url, headers, params);
            log.info("出港：卡车称重接口返回结果:{}",jsonObject);
            verificationRes = JSONObject.parseObject(jsonObject.toString(), VerificationRes.class);
            if("false".equalsIgnoreCase(verificationRes.getRESULT())) {
                res.setCode(500);
                res.setMsg(verificationRes.getMESSAGE());
                //退出登录
                this.LogOff(gatherData.getVE_LICENSE_NO());
                return res;
            }
            if(jsonObject != null && jsonObject.toString().contains("ErrorCode")) {
                res.setCode(500);
                res.setMsg(jsonObject.getString("Message"));
                //退出登录
                this.LogOff(gatherData.getVE_LICENSE_NO());
                return res;
            }
        } catch (Exception e) {
            String message = e.getMessage();
            if(message.contains("Message")) {
                message = message.substring(message.indexOf("Message\":\"")+10,message.indexOf("\"}]"));
            }
            log.info("出港：卡车称重接口报错:{}",message);
            if(message != null && message.contains("Read timed out;")) {
                UCINO = sdf.format(new Date());
                body = "{\"TRUCK_NO\": \""+TRUCK_NO+"\", \"PLACE_CODE\": \""+PLACE_CODE+"\", \"WEIGHT\":"+WEIGHT+", \"THR_FLAG\":\""+THIRDFLAG+"\"}";
                params = JSONObject.parseObject(body);
                try {
                    JSONObject jsonObject2 = RestUtil.post(url, headers, params);
                    log.info("出港2：卡车称重接口返回结果:{}",jsonObject2);
                    if(jsonObject2 != null && jsonObject2.toString().contains("ErrorCode")) {
                        res.setCode(500);
                        res.setMsg(jsonObject2.getString("Message"));
                        return res;
                    }
                    res.setCode(200);
                    res = this.GetOutReport(gatherData,THIRDFLAG);
                } catch (Exception e2) {
                    String message2 = e2.getMessage();
                    if(message2.contains("Message")) {
                        message2 = message2.substring(message2.indexOf("Message\":\"")+10,message2.indexOf("\"}]"));
                    }
                    log.info("出港2：卡车称重接口报错:{}",message2);
                    if(message2 != null && message2.contains("不存在过磅信息或有多条不完整记录")) {
                        res.setCode(200);
                        res = this.GetOutReport(gatherData,THIRDFLAG);
                        return res;
                    }
                    res.setCode(500);
                    res.setMsg(message2);
                    return res;
                }
            }
            res.setCode(500);
            res.setMsg(message);
            return res;
        }
        res.setCode(200);
        res = this.GetOutReport(gatherData,THIRDFLAG);
        return res;
    }

    /** 进港：卡车放行报告接口 */
    public ResponseBo GetInReport(GatherData gatherData, String THIRDFLAG) {
        ResponseBo res = new ResponseBo();
        res.setJsStatus(gatherData.getJsStatus());
        res.setContainerStatus(gatherData.getContainerStatus());
        //车牌号(必输)
        String TRUCK_NO = gatherData.getVE_LICENSE_NO();
        //过磅地点(必输)
        String PLACE_CODE =gatherData.getPlaceCode();
        //道口号(必输)
        String GATE_NO = gatherData.getGateNo();

        //请求地址
        String url = configComponent.getInReportUrl();
        //请求体
        String body = "{\"TRUCK_NO\": \""+TRUCK_NO+"\", \"PLACE_CODE\": \""+PLACE_CODE+"\", \"THR_FLAG\":\""+THIRDFLAG+"\"}";

        JSONObject params = JSONObject.parseObject(body);
        log.info("进港：卡车放行报告接口url:{}",url);
        log.info("进港：卡车放行报告接口params:{}",params);
        try {
            HttpHeaders headers=  HEADERS_MAP.get(gatherData.getVE_LICENSE_NO());
            JSONObject jsonObject = RestUtil.post(url, headers, params);
            log.info("进港：卡车放行报告接口返回结果:{}",jsonObject);
            if(jsonObject != null && jsonObject.toString().contains("ErrorCode")) {
                res.setCode(500);
                res.setMsg(jsonObject.getString("Message"));
                return res;
            }
        } catch (Exception e) {
            String message = e.getMessage();
            if(message.contains("Message")) {
                message = message.substring(message.indexOf("Message\":\"")+10,message.indexOf("\"}]"));
            }
            log.info("进港：卡车放行报告接口报错:{}",message);
            res.setCode(500);
            res.setMsg(message);
            return res;
        }
        res.setJsStatus(1);
        res.setCode(200);
        res.setMsg("散货接口通过");
        //退出登录
        this.LogOff(gatherData.getVE_LICENSE_NO());
        return res;
    }

    /** 出港：卡车放行报告接口 */
    public ResponseBo GetOutReport(GatherData gatherData, String THIRDFLAG) {
        ResponseBo res = new ResponseBo();
        res.setJsStatus(gatherData.getJsStatus());
        res.setContainerStatus(gatherData.getContainerStatus());
        //车牌号(必输)
        String TRUCK_NO = gatherData.getVE_LICENSE_NO();
        //过磅地点(必输)
        String PLACE_CODE = gatherData.getPlaceCode();
        //道口号(必输)
        String GATE_NO = gatherData.getGateNo();

        //请求地址
        String url = configComponent.getOutReportUrl();
        //请求体
        String body = "{\"TRUCK_NO\": \""+TRUCK_NO+"\", \"PLACE_CODE\": \""+PLACE_CODE+"\", \"THR_FLAG\":\""+THIRDFLAG+"\"}";

        JSONObject params = JSONObject.parseObject(body);
        log.info("出港：卡车放行报告接口url:{}",url);
        log.info("出港：卡车放行报告接口params:{}",params);
        VerificationRes verificationRes = new VerificationRes();
        try {
            HttpHeaders headers=  HEADERS_MAP.get(gatherData.getVE_LICENSE_NO());
            JSONObject jsonObject = RestUtil.post(url, headers, params);
            log.info("出港：卡车放行报告接口返回结果:{}",jsonObject);
            verificationRes = JSONObject.parseObject(jsonObject.toString(), VerificationRes.class);
            if("false".equalsIgnoreCase(verificationRes.getRESULT())) {
                res.setCode(500);
                res.setMsg(verificationRes.getMESSAGE());
                //退出登录
                this.LogOff(gatherData.getVE_LICENSE_NO());
                return res;
            }
            if(jsonObject != null && jsonObject.toString().contains("ErrorCode")) {
                res.setCode(500);
                res.setMsg(jsonObject.getString("Message"));
                return res;
            }
        } catch (Exception e) {
            String message = e.getMessage();
            if(message.contains("Message")) {
                message = message.substring(message.indexOf("Message\":\"")+10,message.indexOf("\"}]"));
            }
            log.info("出港：卡车放行报告接口报错:{}",message);
            res.setCode(500);
            res.setMsg(message);
            return res;
        }
        res.setJsStatus(1);
        res.setCode(200);
        res.setMsg("散货接口通过");
        /*ResponseBo bo = this.OutWeightReport(gatherData, THIRDFLAG);
        res.setNotes(bo.getNotes());*/
        //退出登录
        this.LogOff(gatherData.getVE_LICENSE_NO());
        RedisUtil.del("数据交换中心:"+gatherData.getVE_LICENSE_NO());
        //THRFLAG_MAP.remove(gatherData.getVE_LICENSE_NO());
        return res;
    }

    /** 进港：获取卡车进港称重报告(打印小票) */
    public ResponseBo InWeightReport(GatherData gatherData, String THIRDFLAG) {
        ResponseBo res = new ResponseBo();
        res.setJsStatus(gatherData.getJsStatus());
        res.setContainerStatus(gatherData.getContainerStatus());
        //车牌号(必输)
        String TRUCK_NO = gatherData.getVE_LICENSE_NO();
        //过磅地点(必输)
        String PLACE_CODE = gatherData.getPlaceCode();
        //道口号(必输)
        String GATE_NO = gatherData.getGateNo();

        HttpHeaders headers=  HEADERS_MAP.get(gatherData.getVE_LICENSE_NO());
        //请求地址
        String url = configComponent.getInWeightReportUrl() + "?TRUCK_NO="+TRUCK_NO + "&PLACE_CODE="+PLACE_CODE + "&THIRDFLAG="+THIRDFLAG + "&GATE_NO="+GATE_NO;
        JSONObject jsonObject = RestUtil.get(url, headers);
        if(jsonObject != null && jsonObject.toString().contains("ErrorCode")) {
            res.setCode(500);
            res.setMsg(jsonObject.getString("Message"));
            return res;
        }
        res.setCode(200);
        res.setMsg("获取进港小票成功");
        res.setNotes(jsonObject.toJSONString());
        return res;
    }

    /** 出港：获取卡车出港称重报告(打印小票) */
    public ResponseBo OutWeightReport(GatherData gatherData, String THIRDFLAG) {
        ResponseBo res = new ResponseBo();
        res.setJsStatus(gatherData.getJsStatus());
        res.setContainerStatus(gatherData.getContainerStatus());
        //车牌号(必输)
        String TRUCK_NO = gatherData.getVE_LICENSE_NO();
        //过磅地点(必输)
        String PLACE_CODE = gatherData.getPlaceCode();
        //道口号(必输)
        String GATE_NO = gatherData.getGateNo();

        //请求地址
        String url = configComponent.getOutWeightReportUrl() + "?TRUCK_NO="+TRUCK_NO + "&PLACE_CODE="+PLACE_CODE + "&THIRDFLAG="+THIRDFLAG + "&GATE_NO="+GATE_NO;
        HttpHeaders headers=  HEADERS_MAP.get(gatherData.getVE_LICENSE_NO());
        JSONObject jsonObject = RestUtil.get(url, headers);
        if(jsonObject != null && jsonObject.toString().contains("ErrorCode")) {
            res.setCode(500);
            res.setMsg(jsonObject.getString("Message"));
            return res;
        }
        log.info("获取出港小票成功:\n{}",jsonObject.toJSONString());
        res.setMsg("获取出港小票成功");
        res.setNotes(jsonObject.toJSONString());
        return res;
    }

}
