package com.ruoyi.web.service;

import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.web.backgroud.client.BackgroundClientNetty;
import com.ruoyi.web.bo.ResponseBo;
import com.ruoyi.web.config.AesEncryptUtil;
import com.ruoyi.web.config.ConfigComponent;
import com.ruoyi.web.config.Md5Utils;
import com.ruoyi.web.config.MessageXML;
import com.ruoyi.web.entity.Customs;
import com.ruoyi.web.entity.CustomsRes;
import com.ruoyi.web.entity.GatherData;
import com.ruoyi.web.entity.Release;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

/**
 * Netty客户端发送81报文
 */
@Service
public class SendSocket81Service {

    private static final HttpHeaders headers = new HttpHeaders();
    @Autowired
    private ConfigComponent configComponent;

    @Resource
    private BackgroundClientNetty client;

    public void sendSocket(GatherData gatherData) {
        ResponseBo res = new ResponseBo();
        String xml = MessageXML.get81Xml(gatherData);
        client.send(xml);
    }

    /** 采集数据通过验证后，调海关接口，发送json格式的采集数据 */
    public ResponseBo sendGatherData(GatherData gatherData) {
        ResponseBo res = new ResponseBo();
        res.setContainerStatus(0);
        //采集数据转报文格式字符串
        String Content = MessageXML.gatherXml(gatherData);
        //加密报文字符串
        String encryptContent = "";
        try {
            encryptContent = AesEncryptUtil.encrypt(Content);
        } catch (Exception e) {
            System.out.println("----------------------------------数据加密错误:\n"+e.getMessage());
        }

        Customs customs = new Customs();
        customs.setContent(encryptContent);

        String guid = customs.getGuid();
        String token = Md5Utils.hash(guid);
        String tokenStr = "{\"token\":\"" + token + "\"}";
        //请求地址
        String url = configComponent.getGatherUrl();
        //拼接 url 参数
        JSONObject variables = JSONObject.parseObject(tokenStr);
        //请求体
        JSONObject params = JSONObject.parseObject(JSONObject.toJSONString(customs));

        /*JSONObject result = RestUtil.post(url, headers, variables, params);
        //返回放行结果对象
        CustomsRes customsRes = JSONObject.parseObject(result.toString(), CustomsRes.class);*/
        CustomsRes customsRes = new CustomsRes();
        customsRes.setOP_REASON("放行");

        res.setCode(200);
        res.setMsg(customsRes.getOP_REASON());
        if(!"放行".equals(customsRes.getOP_REASON())) {
            res.setCode(500);
            return res;
        }
        res.setContainerStatus(1);
        return res;
    }

    /** 车辆通过卡口后，调海关接口，发送放行结果信息给海关系统 */
    public ResponseBo sendReleaseData(Release release) {
        ResponseBo res = new ResponseBo();
        //放行结果转报文格式字符串
        String Content = MessageXML.ReleaseResXml(release);
        //加密报文字符串
        String encryptContent = "";
        try {
            encryptContent = AesEncryptUtil.encrypt(Content);
        } catch (Exception e) {
            System.out.println("----------------------------------数据加密错误:\n"+e.getMessage());
        }

        Customs customs = new Customs();
        customs.setContent(encryptContent);

        String guid = customs.getGuid();
        String token = Md5Utils.hash(guid);
        String tokenStr = "{\"token\":\"" + token + "\"}";
        //请求地址
        String url = configComponent.getReleaseUrl();
        //拼接 url 参数
        JSONObject variables = JSONObject.parseObject(tokenStr);
        //请求体
        JSONObject params = JSONObject.parseObject(JSONObject.toJSONString(customs));

        /*JSONObject result = RestUtil.post(url, headers, variables, params);
        //返回放行结果对象(暂定返回0x31报文)
        CustomsRes customsRes = JSONObject.parseObject(result.toString(), CustomsRes.class);*/

        CustomsRes customsRes = new CustomsRes();
        customsRes.setOP_REASON("放行");

        res.setCode(200);
        res.setMsg(customsRes.getOP_REASON());
        if(!"放行".equals(customsRes.getOP_REASON())) {
            res.setCode(500);
            return res;
        }
        return res;
    }

}
